require 'open-uri'
require 'aws-sdk-s3'
require 'openssl'

module NxtHerokuDatabaseBackup
  class Manager
    include NxtInit

    attr_init :pass_phrase,
              :salt,
              success_callback: -> { -> { true } },
              environment: -> { default_environment },
              backup_name: -> { default_backup_name },
              backup_tmp_path: -> { default_backup_tmp_path },
              uploader: -> { build_uploader }


    def backup_latest_dump
      error = nil
      download_backup
      encrypt_backup
      upload_backup
      execute_success_callback
    rescue StandardError => err
      error = err
    ensure
      [backup_tmp_path, encrypted_backup_tmp_path].each do |path|
        File.delete(path) if File.exist?(path)
      end

      raise error if error
    end

    private

    def build_uploader
      Uploader::S3.build(
        bucket_name: upload_destination,
        file_path: encrypted_backup_tmp_path,
        backup_name: backup_name
      )
    end

    def upload_backup
      uploader.call
    end

    def encrypt_backup
      data = NxtHerokuDatabaseBackup::Encrypter.new(
        pass_phrase: pass_phrase,
        salt: salt,
        data: File.read(backup_tmp_path)
      ).call

      File.open(encrypted_backup_tmp_path, 'wb') { |f| f.puts(data) }
    end

    def download_backup
      IO.copy_stream(open(database_backup_url), backup_tmp_path)
    end

    def encrypted_backup_tmp_path
      "#{backup_tmp_path}.encrypted"
    end

    def execute_success_callback
      success_callback.call
    end

    def default_backup_name
      "#{heroku_app_name}-#{Time.now.strftime('%Y-%m-%d-%H-%M-%S')}.dump"
    end

    def default_backup_tmp_path
      "/tmp/#{backup_name}"
    end

    def database_backup_url
      @database_backup_url ||= begin
        run_heroku_cli_command "pg:backups:url"
      end
    end

    def run_heroku_cli_command(command)
      `heroku #{command} -a #{heroku_app_name}`
    end

    def heroku_app_name
      @heroku_app_name ||= ENV.fetch('HEROKU_APP_NAME')
    end

    def upload_destination
      @upload_destination ||= "#{heroku_app_name}-db-backups"
    end

    def default_environment
      Object.const_defined?('HerokuEnv') && HerokuEnv.env || 'development'
    end
  end
end
